@@ -19,6 +19,7 @@ from aiohomekit.controller.ble.manufacturer_data import (
     HomeKitEncryptedNotification,
 )
 from aiohomekit.controller.ble.pairing import BlePairing
+from aiohomekit.utils import asyncio_timeout
 
 from .discovery import BleDiscovery
 
@@ -153,7 +154,8 @@ class BleController(AbstractController):
         future = asyncio.Future()
         self._ble_futures_by_id.setdefault(device_id, []).append(future)
         try:
-            return await asyncio.wait_for(future, timeout=timeout)
+            async with asyncio_timeout(timeout):
+                return await future
         except asyncio.TimeoutError:
             logger.debug(
                 "Timed out after %s waiting for discovery with hkid %s",
@@ -189,7 +191,8 @@ class BleController(AbstractController):
         future = asyncio.Future()
         self._ble_futures.setdefault(address, []).append(future)
         try:
-            return await asyncio.wait_for(future, timeout=timeout)
+            async with asyncio_timeout(timeout):
+                return await future
         except asyncio.TimeoutError:
             logger.debug(
                 "Timed out after %s waiting for discovery with address %s",
