@@ -13,10 +13,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
+from collections.abc import Sequence
 from dataclasses import dataclass, field
 import enum
 import struct
-from typing import Any, Optional, Sequence, Union
+from typing import Any, Optional, Union
 
 from aiohomekit.tlv8 import TLVStruct, tlv_entry, u8, u16, u128
 
